#!/usr/bin/env python
#
#       base.py
#       
#       Copyright 2009 Gordon Allott <gordallott@gmail.com>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
import os, sys
from gamelib import render, input, audio


""" The idea with an AI is to create an AI that reacts to collisions generated
I want the collisions generated to be fairly advanced, basically what the collision
code will do is look for a function in the AI object called
collision_with_<objecttype>()
replacing <objecttype> with whatever the type of the object that you collided with
is, if its found it calls it, if not, it ignores the collision for this AI.
"""


# My stats at the end of a test playthough, playing fairly balanced
# health -25
# fun 16
# 

def propertyd(function):
    keys = 'fget', 'fset', 'fdel'
    func_locals = {'doc':function.__doc__}
    def probe_func(frame, event, arg):
        if event == 'return':
            locals = frame.f_locals
            func_locals.update(dict((k, locals.get(k)) for k in keys))
            sys.settrace(None)
        return probe_func
    sys.settrace(probe_func)
    function()
    return property(**func_locals)


class Statistics(object):
    
    """ tracks our statistics """
    
    _fun = 0
    _love = 0
    _health = 0
    _work = 0
    _beers = 0
    _time = 0
    
    _attention = None
    
    _renderables = None
    _textures = None
    _player_loc = (0.0, 0.0)
    _bubble = None
    

    win_love = 17
    win_happy = 10
    is_married = False
    
    is_busy = False
 
    @propertyd 
    def bubble():
        """ the position of our player """
        def fget(self): return self.__class__._bubble
        def fset(self, value): 
            self.__class__._bubble = value
    
    @propertyd 
    def player_loc():
        """ the position of our player """
        def fget(self): return self.__class__._player_loc
        def fset(self, value): 
            self.__class__._player_loc = value
    
    @propertyd 
    def renderables():
        """ the position of our item """
        def fget(self): return self.__class__._renderables
        def fset(self, value): 
            self.__class__._renderables = value
            
    @propertyd 
    def textures():
        """ the position of our item """
        def fget(self): return self.__class__._textures
        def fset(self, value): 
            self.__class__._textures = value


    @propertyd 
    def time():
        """ the position of our item """
        def fget(self): return self.__class__._time
        def fset(self, value): 
            self.__class__._time = value
            print 'time is now at', self.__class__._time
            
                
    @propertyd 
    def fun():
        """ the position of our item """
        def fget(self): return self.__class__._fun
        def fset(self, value): 
            mod = value - self.__class__._fun 
            self.__class__._fun = value
            if mod > 0:
                self.new_icon('happy')
            else:
                self.new_icon('badhappy')
            print 'fun is now at', self.__class__._fun
            
        
    @propertyd 
    def love():
        """ the position of our item """
        def fget(self): return self.__class__._love
        def fset(self, value):  
            mod = value - self.__class__._love
            self.__class__._love = value
            if mod > 0:
                self.new_icon('love')
            else:
                self.new_icon('badlove')
            print 'love is now at', self.__class__._love
    
    @propertyd 
    def health():
        """ the position of our item """
        def fget(self): return self.__class__._health
        def fset(self, value):  
            mod = value - self.__class__._health
            self.__class__._health = value
            if mod > 0:
                self.new_icon('health')
            else:
                self.new_icon('badhealth')
            print 'health is now at', self.__class__._health

    @propertyd 
    def work():
        """ the position of our item """
        def fget(self): return self.__class__._work
        def fset(self, value):  
            mod = value - self.__class__._work
            self.__class__._work = value
            if mod > 0:
                self.new_icon('work')
            else:
                self.new_icon('badwork')
            print 'work is now at', self.__class__._work


    @propertyd 
    def beers():
        """ the position of our item """
        def fget(self): return self.__class__._beers
        def fset(self, value): 
            self.__class__._beers = value
            print 'Drank this many beers', self.__class__._beers

    @propertyd 
    def attention():
        """ the position of our item """
        def fget(self): return self.__class__._attention
        def fset(self, value): self.__class__._attention = value
        
    
    def calc_happy(self):
        ''' returns < 0 if unhappy, 0 if neuatral and 1 if happy '''
            #love: 21
    
        happy_points = 4
        if self.love < 10:
            happy_points -= 1
            
        if self.love < 5:
            happy_points -= 1
        
        if self.health < -5:
            happy_points -= 1
        
        if self.health < -10:
            happy_points -= 1
        
        if self.fun < 10:
            happy_points -= 1
            
        if self.work < 10:
            happy_points -= 1
        
        return happy_points #happy happy joy joy!
    
    def __init__(self):
        self.rcontext = render.render_context()
        
        if self.renderables == None:
            self.renderables = []
            
        if self.textures == None:
            self.textures = {}
            texture = self.rcontext.create_texture('graphics/status/love.png')
            self.textures['love'] = texture
            texture = self.rcontext.create_texture('graphics/status/happy.png')
            self.textures['happy'] = texture
            texture = self.rcontext.create_texture('graphics/status/health.png')
            self.textures['health'] = texture
            texture = self.rcontext.create_texture('graphics/status/work.png')
            self.textures['work'] = texture
            
            texture = self.rcontext.create_texture('graphics/status/badlove.png')
            self.textures['badlove'] = texture
            texture = self.rcontext.create_texture('graphics/status/badhappy.png')
            self.textures['badhappy'] = texture
            texture = self.rcontext.create_texture('graphics/status/badhealth.png')
            self.textures['badhealth'] = texture
            texture = self.rcontext.create_texture('graphics/status/badwork.png')
            self.textures['badwork'] = texture
            
            texture = self.rcontext.create_texture('graphics/thought_bubble.png')
            self.textures['bubble'] = texture
            
            self.bubble = render.BasicBox()
            self.bubble.color = (1.0, 1.0, 1.0, 0.6)
            self.bubble.scale = (48, 48)
            self.bubble.add_texture('bubble', self.textures['bubble'])
            self.bubble.set_texture('bubble')
            
    def new_icon(self, type):
        renderable = render.BasicBox()
        renderable.color = (1.0, 1.0, 1.0, 1.0)
        renderable.scale = (24, 24)
        renderable.add_texture(type, self.textures[type])
        renderable.set_texture(type)
        
        self.renderables += [(renderable, 1.0, 0.0, False)]
    
            
    def tick(self):
        new_data = []
        
        space_left = 1024
        #if self.renderables:
            #print self.renderables
        for (renderable, alpha, height, started) in self.renderables:
            
            if started:
                height += 60.0 / 60.0
                alpha -= 1.0 / 60.0
                
                space_left = min(space_left, height - 8)
                
                if alpha > 0.0:
                    new_data += [(renderable, alpha, height, started)]
                    
            else:
                #we havn't started yet, check to see if we have enough height
                #to start
                if space_left >= 20:
                    space_left = 0
                    new_data += [(renderable, alpha, height, True)]
                else:
                    new_data += [(renderable, alpha, height, False)]
            
        self.renderables = new_data
        
    def draw(self, base_pos=(0.0, 0.0)):
        
        for renderable, alpha, height, started in self.renderables:
            if started:
                #print 'status!', renderable, alpha, height, base_pos
                renderable.color = (1.0, 1.0, 1.0, alpha)
                newpos = (self.player_loc[0], self.player_loc[1] + height + 64)
                self.rcontext.move_to(*newpos)
                renderable.draw()
                
        #draw our thought bubble and icon
        newpos = (self.player_loc[0] - 32, self.player_loc[1] + 64)
        self.rcontext.move_to(*newpos)
        self.bubble.draw()
        
        if self.attention != None:
            if self.attention.renderable != None:
                newpos = (newpos[0] - 0, newpos[1] + 0)
                self.rcontext.move_to(*newpos)
                prescale = self.attention.renderable.scale
                precolor = self.attention.renderable.color
                color = None
                if len(precolor) == 4:
                    color = (precolor[0], precolor[1], precolor[2], precolor[3] * 0.8)
                else:
                    color = (precolor[0], precolor[1], precolor[2], 0.8)
                    
                #FIXME! add in aspect correct scaling
                scalar = None
                if prescale[0] > prescale[1]:
                    scalar = 20.0 / prescale[0]
                else:
                    scalar = 20.0 / prescale[1]
                
                
                self.attention.renderable.scale = (prescale[0] * scalar, prescale[1] * scalar)
                self.attention.renderable.color = color
                self.attention.renderable.draw()
                self.attention.renderable.scale = prescale
                self.attention.renderable.color = precolor
        
        
        
        

class AI_manager(object):
    
    """ attempts to manage AI's somewhat, basically allows for things like ai's
    to get a special callback once per frame and such
    """
    ai_callbacks = None
    def __init__(self):
        if self.__class__.ai_callbacks == None:
            self.__class__.ai_callbacks = {}
            
    def reset(self):
        self.__class__.ai_callbacks = {}
    
    def add_callback(self, ai):
        """ adds an ai to a per-tick callback, make the callback return false
        to remove the AI from the callback, else return true
        """
        self.__class__.ai_callbacks[ai] = ai
        
    def do_tick(self):
        removals = []
        for name, value in self.__class__.ai_callbacks.iteritems():
            retval = value.ai_tick_call()
            if not retval:
                removals += [name]
                
        for name in removals:
            self.__class__.ai_callbacks.pop(name)
            

class AI(object):
    
    _rcontext = None
    _acontext = None
    _input = None
    
    parent_entity = None
    call_on_tick = False
    triggered = False
    
    @propertyd 
    def rcontext():
        """ the position of our item """
        def fget(self): return self.__class__._rcontext
        def fset(self, value): self.__class__._rcontext = value
    
    @propertyd 
    def acontext():
        """ the position of our item """
        def fget(self): return self.__class__._acontext
        def fset(self, value): self.__class__._acontext = value
    
    @propertyd 
    def input():
        """ the position of our item """
        def fget(self): return self.__class__._input
        def fset(self, value): self.__class__._input = value
    
    def __init__(self, parent_entity, **kwargs):
        if not self.rcontext:
            self.rcontext = render.render_context()
        if not self.acontext:
            pass
        if not self.input:
            self.input = input.Input()
            
        self.parent_entity = parent_entity
        
    def tick(self):
        pass
    

class ScreenSpace(AI):
    
    call_on_tick = True
    camera = None
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.camera = render.Camera()
        
    def tick(self):
        x, y = self.camera.current_position
        self.parent_entity.position = (x, -y)
    
    

class Jason(AI):
    
    """ Right, this AI is gonna get sort of complicated i think huh..
    try and re-use functions when you can
    """
    
    call_on_tick = True
    _statistics = {}
    
    _entity_attention = None
    
    walk_speed = 3.56

    
    @propertyd 
    def statistics():
        """ foobar """
        def fget(self): return self.__class__._statistics
        def fset(self, value): self.__class__._statistics = value
        
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        
        if not self.statistics:
            self.statistics = Statistics()
        
        self.reload_walk = False #if this is true, reload our walk_anim if the current animation finishes
            
        self.camera = render.Camera()
        self.camera.track_object(self.parent_entity, mod=(100, 0))
        
        self.parent_entity.renderable.set_texture('walk_anim')
        self.busy_tick = 0
    
        self.acontext = audio.audio_context()
        self.has_ring = False
        self.has_flower = False
        
        self.do_end_animation = False
        
        self.walk_type = 'walk_anim'
        
    def collide_with_generic(self, entity):
        #print self, 'Collided with', entity
        pass
        
    def tick(self):
        x, y = self.parent_entity.position
        x += self.walk_speed
        #update statistics
        self.statistics.player_loc = (x, y)

        self.parent_entity.position = (x, y)
        
        if self.statistics.calc_happy() > 1:
            self.walk_type = 'walk_strut'
        if self.statistics.calc_happy() < 1:
            self.walk_type = 'walk_shuffle'
        else:
            self.walk_type = 'walk_anim'

        
        if hasattr(self.parent_entity.renderable.texture, 'finished_animation'):
            if self.parent_entity.renderable.texture.finished_animation:
                
                if self.do_end_animation:
                    if self.reload_walk == 'angry_start':
                        self.reload_walk = 'angry'
                    
                
                if self.reload_walk:
                    self.parent_entity.renderable.texture.reset()
                    self.parent_entity.renderable.set_texture(self.reload_walk)
                    self.reload_walk = False
                    
                
                    
        #if self.busy_tick > 0:
            #self.statistics.is_busy = True
            #self.statistics.attention = None
            #self.busy_tick -= 1
        #else:
            #self.statistics.is_busy = False
        
    def collision_with_test_hitbox(self, collision_entity):
        #print collision_entity.type, collision_entity
        pass
        
    #def collision_with_screen_space(self, collision_entity):
    #    pass
    #    print 'collided with screen space!'
       
    def collision_with_pub_beer(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            # this is what we were thinking of :) perhaps replace this with a decorator?
            
            self.statistics.fun += 1
            self.statistics.health -= 1
            self.statistics.beers += 1
            self.statistics.time -= 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            
            #load our drinking texture
            self.reload_walk = self.walk_type
            
            self.parent_entity.renderable.set_texture('drink_beer')
            self.busy_tick = 30
            collision_entity.destroy()
            
    
    def collision_with_juice(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.fun -= 1
            self.statistics.health += 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            
            self.reload_walk = self.walk_type
            self.parent_entity.renderable.set_texture('drink_juice')
            self.busy_tick = 30
            collision_entity.destroy()
            
    def collision_with_wine(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.fun += 1
            self.statistics.health -= 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            
            self.reload_walk = self.walk_type
            self.parent_entity.renderable.set_texture('drink_wine')
            self.busy_tick = 30
            collision_entity.destroy()
            
            
    def collision_with_soap(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.health += 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            
            #self.reload_walk = 'walk_anim'
            #self.parent_entity.renderable.set_texture('drink_wine')
            self.busy_tick = 30
            collision_entity.destroy()
            self.parent_entity.renderable.set_texture(self.walk_type)
            
    def collision_with_toothbrush(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.health += 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            
            #self.reload_walk = 'walk_anim'
            #self.parent_entity.renderable.set_texture('drink_wine')
            self.busy_tick = 30
            self.parent_entity.renderable.set_texture(self.walk_type)
            collision_entity.destroy()
            
    def collision_with_tv(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.health -= 1
            self.statistics.fun += 1
            
            collision_entity.ai.run_tv()
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.parent_entity.renderable.set_texture(self.walk_type)
            #self.reload_walk = 'walk_anim'
            #self.parent_entity.renderable.set_texture('drink_wine')
            
            #self.busy_tick = 30
            #collision_entity.destroy()
        
            

    def collision_with_pub_guitar(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            # this is what we were thinking of :) perhaps replace this with a decorator?
            
            self.statistics.fun += 5
            self.statistics.health -= 2
            self.statistics.love += 1
            self.statistics.time -= 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.busy_tick = 30
            
            #FIXME, zb provided a take animation and a loop one, when you ahve time
            #make this take it then loop? or.. wait.. no.. i need to work on this
            self.reload_walk = self.walk_type
            self.parent_entity.renderable.set_texture('guitar_take')
            
            self.guitar_sample = audio.audio_sample('dobroide__blues.ogg')
            self.guitar_sample.play_sample()
            collision_entity.destroy()

    def collision_with_coffee_pot(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            # this is what we were thinking of :) perhaps replace this with a decorator?
            
            self.statistics.fun += 1
            self.statistics.health -= 1
            self.statistics.work += 1
            self.statistics.time += 1
            
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.busy_tick = 30
            
            #load our drinking texture
            self.reload_walk = self.walk_type
            self.parent_entity.renderable.set_texture('drink_coffee')
            collision_entity.destroy()
            
                        
    def collision_with_office_drone(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            if self.statistics.attention.ai.can_activate:
                collision_entity.ai.take_paper()
                self.statistics.fun -= 1
                self.statistics.work += 1
                self.statistics.time -= 1
                
                self.statistics.attention.ai.triggered = True
                self.statistics.attention = None
                
                self.reload_walk = self.walk_type
                self.parent_entity.renderable.set_texture('work')
                self.busy_tick = 30
                
                
    def collision_with_ring(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.love += 1
            self.statistics.work -= 3
            
            
            self.has_ring = True
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            collision_entity.destroy()
            self.parent_entity.renderable.set_texture(self.walk_type)
            
            
    def collision_with_flower(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.has_flower = True
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            collision_entity.destroy()
            self.parent_entity.renderable.set_texture(self.walk_type)
                
    def collision_with_spouse_courting(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            if self.statistics.attention.ai.can_activate:
                self.statistics.fun += 1
                self.statistics.love += 1
                self.statistics.work -= 1
                
                if self.has_ring:
                    self.statistics.attention.ai.start_walk(self, True)
                    self.statistics.__class__.is_married = True
                else:
                    self.statistics.attention.ai.start_walk(self, False)
                
                self.statistics.attention.ai.can_activate = False
                self.statistics.attention.ai.triggered = True
                self.statistics.attention = None
                self.busy_tick = 90
                self.parent_entity.renderable.set_texture(self.walk_type)
                

    def collision_with_spouse_sitting(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            if self.has_flower == True:
                self.statistics.love += 1
                
            self.statistics.love += 1
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.parent_entity.renderable.set_texture(self.walk_type)
            
    def collision_with_baby(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.love += 1
            self.statistics.fun -= 1
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.parent_entity.renderable.set_texture(self.walk_type)
            
    def collision_with_child_play(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.love += 1
            self.statistics.fun += 1
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.parent_entity.renderable.set_texture(self.walk_type)
            
            
    def collision_with_child_chair(self, collision_entity):
        if self.statistics.attention == collision_entity and not collision_entity.ai.triggered:
            self.statistics.love += 1
            self.statistics.work += 1
            self.statistics.attention.ai.triggered = True
            self.statistics.attention = None
            self.parent_entity.renderable.set_texture(self.walk_type)
            
            
    def collision_with_end_start(self, collision_entity):
        if not collision_entity.ai.triggered:
            collision_entity.ai.triggered = True
            self.statistics.attention = None
            self.do_end_animation = True
            if self.statistics.calc_happy() < 1:
                self.reload_walk = 'angry_start'
                
            self.parent_entity.renderable.set_texture('sitting')
            self.walk_speed = 0.0
            self.parent_entity.position = collision_entity.position
            

class spouse_courting(AI):
    """ for the courting spouse we want her to stand in the pub, not doing much
    and then get her to follow you for a bit if you click her
    """
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('stand')
        self.can_activate = True
        self.give_points = False
        self.following = 0
        self.do_engagement = 0
        self.ai_manager = AI_manager()
        self.set_tex = True
        
    def start_walk(self, ai, do_engagement):
        #walk alongside the ai given
        self.walker = ai
        self.ai_manager.add_callback(self)
        
        if do_engagement:
            self.do_engagement = True
            self.following = 400
            self.set_tex = False
        else:
            self.following = 90
            self.set_tex = False
        
        self.parent_entity.renderable.set_texture('walk_anim')
        
        
    def ai_tick_call(self):
        pos = self.walker.parent_entity.position
        self.parent_entity.position = (pos[0] + 64, pos[1])
        self.following -= 1
        
        if self.following > 0:
            return True
        else:
            if self.do_engagement:
                if self.set_tex == False:
                    self.parent_entity.renderable.set_texture('proposal')
                    self.walker.parent_entity.renderable.set_texture('proposal')
                    self.walker.reload_walk = False
                    self.set_tex = True
                return True
            else:    
                self.parent_entity.renderable.set_texture('stand')
                return False
      
      
class baby(AI):
    
    sample = None
    played = False
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        
        self.played_sample = None
        if not self.__class__.sample:
            self.__class__.sample = audio.audio_sample('Erdie__baby2.ogg')
        
        self.played = False
        
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
            self.no_play = True
        else:
            self.no_play = False
        
    def collision_with_screen_space(self, collision):
        if not self.played and not self.no_play:
            self.played = True
            self.__class__.sample.play_sample()

class ring(AI):
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        

class flower(AI):
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class spouse_sitting(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.time <= 0:
                self.parent_entity.destroy()
        
        
class spouse_sleeping(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.time > 0:
                self.parent_entity.destroy()
        
        
class spouse_endsit(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.love <= 10:
                self.parent_entity.destroy()
                
class child_end(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.love <= 10:
                self.parent_entity.destroy()
        
class child_chair(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.time <= 0:
                self.parent_entity.destroy()
        

class child_play(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.statistics = Statistics()
        self.do_check = False
        if not Statistics().__class__.is_married:
            self.parent_entity.destroy()
        
    def collision_with_screen_space(self, collision):
        if not self.do_check:
            self.do_check = True
            if self.statistics.time <= 0:
                self.parent_entity.destroy()        

class pub_beer(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
    
class pub_guitar(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class coffee_pot(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class wine(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class juice(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class tv(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

    def run_tv(self):
        self.parent_entity.renderable.set_texture('on')

class soap(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class toothbrush(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')

class pub_beer(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')









class end_start(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        #self.parent_entity.renderable.set_texture('default')


class office_drone(AI):
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        
        self.__class__.aimanager = AI_manager()
        self.can_activate = False
        
    def collision_with_jason(self, item):
        self.aimanager.add_callback(self)
        self.parent_entity.renderable.set_texture('give_paper')
        

    def ai_tick_call(self):
        if self.parent_entity.renderable.texture.finished_animation:
            self.can_activate = True
            return False
        
        return True
            
            
    def take_paper(self):
        self.can_activate = False
        self.parent_entity.renderable.set_texture('default')




class act_changer(AI):
    
    _world_callback = None
    
    def add_callback(self, callback):
        self.__class__._world_callback = callback
        
    def collision_with_jason(self, thingy):
        if self.__class__._world_callback:
            self.__class__._world_callback()
            self.__class__._world_callback = None


class sky(AI):
    
    done_change = False
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('evening')
        self.done_change = False
    
    def collision_with_screen_space(self, collision):
        if not self.done_change:
            statistics = Statistics()
            if statistics.time > 0:
                self.parent_entity.renderable.set_texture('evening')
            else:
                self.parent_entity.renderable.set_texture('night')
            self.done_change = True


class credit_result(AI):
    
    done_change = False
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('a')
        self.done_change = False
        
        self.wait_for_move = 140
        self.aimanager = AI_manager()
        
    
    def collision_with_screen_space(self, collision):
        if not self.done_change:
            statistics = Statistics()
            print 'fun', statistics.fun
            print 'love', statistics.love
            print 'health', statistics.health
            print 'work', statistics.work
            print 'beers', statistics.beers
            if statistics.love >= statistics.win_love and statistics.calc_happy() > 0:
                self.parent_entity.renderable.set_texture('d')
            elif statistics.love >= statistics.win_love and statistics.calc_happy() <= 0:
                self.parent_entity.renderable.set_texture('b')
            elif statistics.love < statistics.win_love and statistics.calc_happy() > 0:
                self.parent_entity.renderable.set_texture('c')
            else:
                self.parent_entity.renderable.set_texture('a')
            self.done_change = True
            
            self.aimanager.add_callback(self)
            
    def ai_tick_call(self):
        self.wait_for_move -= 1
        if self.wait_for_move > 0:
            return True
            
        x, y = self.parent_entity.position
        y = min(y, 1024)
        self.parent_entity.position = (x, y+1)
        return True
        
class credits(AI):
    
    done_change = False
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.parent_entity.renderable.set_texture('default')
        self.done_change = False
        
        self.wait_for_move = 140+140
        self.aimanager = AI_manager()
        
    
    def collision_with_screen_space(self, collision):
        if not self.done_change:
            self.done_change = True
            self.aimanager.add_callback(self)
            
    def ai_tick_call(self):
        self.wait_for_move -= 1
        if self.wait_for_move > 0:
            return True
            
        x, y = self.parent_entity.position
        y = min(y, 1024)
        self.parent_entity.position = (x, y+0.5)
        return True
            
            
#Sounds

class schoolbell(AI):
    
    """ Plays our schoolbell sound, Fixme!!! - make it fade away to the left - use timeline callback"""
    
    def __init__(self, *args, **kwargs):
        AI.__init__(self, *args, **kwargs)
        self.acontext = audio.audio_context()
        self.sample = audio.audio_sample('john_Sauter__Electric_school_bell.ogg')
        self.played = False
        
    def collision_with_screen_space(self, collision_entity):
        if not self.played:
            #self.acontext.play_sample(self.sample) 
            self.played = True
        
class audio_ai(AI):

    sample = None
    fade_in = -1.0
    fade_out = -1.0
    
    filename = 'fogma__Bar_atmosphere_Not_So_Busy.ogg'
    
    def __init__(self, *args, **kwargs):
        self.state = 'start'
        if kwargs.has_key('data'):
            self.state = kwargs['data']
            kwargs.pop('data')
             
        AI.__init__(self, *args, **kwargs)
        self.__class__.aid = None
        self.__class__.acontext = audio.audio_context()
        self.__class__.aimanager = AI_manager()
        if not self.__class__.sample: 
            self.__class__.sample = audio.audio_sample(self.filename)
            
        self.activated = False
        
    def collision_with_screen_space(self, collision_entity):
        if self.state == 'start':
            if not self.activated:
                print 'starting %s noise' % (self.filename)
                self.__class__.aid = self.__class__.sample.play_sample(loop=-1) 
                self.__class__.sample.set_distance_angle(0.5, 1.0)
                self.activated = True
                self.fade_in = 1.0
                self.fade_out = -1.0
                self.aimanager.add_callback(self)
        else:
            if not self.activated:
                print 'ending %s noise' % (self.filename)
                if self.__class__.sample:
                    self.__class__.sample.set_distance_angle(0.0, 0.0)
                    self.activated = True
                    self.fade_out = 0.0
                    self.fade_in = -1.0
                    self.aimanager.add_callback(self)
                    
                    #self.__class__.sample.stop_sample()
                    
                self.activated = True
        
        
    def ai_tick_call(self):
        if self.fade_in > 0.0 and self.fade_in <= 1.0:
            self.__class__.sample.set_distance_angle(self.fade_in, self.fade_in)
            self.fade_in -= 2.0 / 30.0
        
            if self.fade_in <= 0.0:
                return False
        
        elif self.fade_out >= 0.0 and self.fade_out < 1.0:
            self.__class__.sample.set_distance_angle(self.fade_out, -self.fade_out)
            self.fade_out += 2.0 / 60.0
        
            if self.fade_out >= 1.0:
                self.__class__.sample.stop_sample()
                return False
        
        return True

class bar_sound(audio_ai):
    
    filename = 'fogma__Bar_atmosphere_Not_So_Busy.ogg'
    
class home_sound(audio_ai):
    
    filename = 'laurent__natural_night_sounds_in_Boquete.ogg'
    
class office_sound(audio_ai):
    
    filename = 'LG__Office02.ogg'

    def __init__(self, *args, **kwargs):
        self.done_refil = False
        audio_ai.__init__(self, *args, **kwargs)

    def collision_with_screen_space(self, collision):
        
        if not self.done_refil and self.state == 'start':
            # we attach some special code here to refil our time thing
            statistics = Statistics()
            statistics.time = 4
            self.done_refil = True
            
        audio_ai.collision_with_screen_space(self, collision)
