#!/usr/bin/env python
#
#       svg_parse.py
#       
#       Copyright 2009 Gordon Allott <gordallott@gmail.com>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import random
from random import random as randfloat
from math import *
from xml.dom import minidom
import os
from collections import deque 
import gamelib.data as data

# I guess what we want out of the svg parser so far is a dict of layers and the 
# objects inside those layers expressed as information

def make_poly(xmlstring):
    
    data = xmlstring.split('M ')[1].split(' z')[0]
    data = data.replace('L', '')
    data = data.split(' ')
    verts = []
    
    for pair in data:
        v1, v2 = pair.split(',')
        v1 = float(v1)
        v2 = float(v2)
        verts.append((v1, v2))
        
    return verts


def hex_to_gl(colorhex):
    colorhex = colorhex.replace('#', '')

    red = colorhex[0] + colorhex[1]
    green = colorhex[2] + colorhex[3]
    blue = colorhex[4] + colorhex[5]

    color = (float(int(red, 16)) / 255.0,
             float(int(green, 16)) / 255.0,
             float(int(blue, 16)) / 255.0,
            )

    return color
    
def get_style(xmlstring):
    #fill:#000000;fill-opacity:1;stroke:none
    
    tmp = xmlstring.split(';')
    styles = {}
    for item in tmp:
        key, value = item.split(':')
        styles[key] = value
    
    value = random.random()
    
    color = (1.0, 1.0, 1.0)
    try:
        color = hex_to_gl(styles['fill'])
    except:
        pass
        
    return {'color': color}
    
def get_layer_info(xmlstring):
    layer_type, layer_name, z = xmlstring.split('_')
    z = int(z.replace('z', ''))
    return layer_type, layer_name, z

def parseXML(filename):
    
    #load the file
    filepath = data.filepath(filename)
    xml_doc = minidom.parse(filepath)
    xmlroot = xml_doc.getElementsByTagName('svg')[0]

    layers = {}
    

    xmllayers = xml_doc.getElementsByTagName('g')
    for xmllayer in xmllayers:
        layer = {}
        layer['id'] = xmllayer.attributes['id'].nodeValue
        layer['items'] = deque() #use a deque for memory niceness
        layer['type'], layer['label'], layer['z'] = get_layer_info(xmllayer.attributes['inkscape:label'].nodeValue)
        
        rects = xmllayer.getElementsByTagName('rect')
        for item in rects:
            rect = {'type': 'rect', 'entdef': None, 'entdata': None}
            rect['id'] = item.attributes['id'].nodeValue
            try:
                rect['style'] = get_style(item.attributes['style'].nodeValue)
            except:
                rect['style'] = get_style('fill:#FFFFFF')
            
            rect['width'] = float(item.attributes['width'].nodeValue)
            rect['height'] = float(item.attributes['height'].nodeValue)
            
            x = float(item.attributes['x'].nodeValue)
            y = 225 - (float(item.attributes['y'].nodeValue) + rect['height'])
            
            rect['x'] = x + (rect['width'] * 0.5)
            rect['y'] = y + (rect['height'] * 0.5)
            
            rect['texture'] = None
            rect['texture_tile'] = False
            
            layer['items'].append(rect)
        
        
        xmlitems = xmllayer.getElementsByTagName('image')
        
        for item in xmlitems:
            rect = {'type': 'rect', 'entdef': None,  'entdata': None}
            rect['id'] = item.attributes['id'].nodeValue
            try:
                rect['style'] = get_style(item.attributes['style'].nodeValue)
            except:
                rect['style'] = get_style('fill:#FFFFFF')
            
            rect['width'] = float(item.attributes['width'].nodeValue)
            rect['height'] = float(item.attributes['height'].nodeValue)
            
            x = float(item.attributes['x'].nodeValue)
            y = 576.0 - (float(item.attributes['y'].nodeValue) + rect['height'])
            
            rect['x'] = x + (rect['width'] * 0.5)
            rect['y'] = y + (rect['height'] * 0.5)
            
            rect['texture'] = item.attributes['xlink:href'].nodeValue
            if rect['texture'].find('/tileable/') > 0:
                rect['texture_tile'] = True
            else:
                rect['texture_tile'] = False
                
            try:
                rect['entdef'] = item.attributes['entdef'].nodeValue
            except:
                pass
                
            try:
                rect['entdata'] = item.attributes['entdata'].nodeValue
            except:
                pass
                
            
            
            layer['items'].append(rect)
            
        layers[layer['id']] = layer
    
    
    for layer in layers.itervalues():
        print 'layer found:', layer['label']
        print 'z-value', layer['z']
        print 'type', layer['type']
        
    return layers
